# A basic linked list implementation.
# Note: Since this file contains two classes, the file
# is named after LinkedList, the more important class.
# 
# Author: David Kauchak
# Date: 2/15/2012

# A node in the linked list
#
class Node
	attr_accessor :next
	attr_reader :value
	
	# create a node with value stored in it
	def initialize(value)
		@value = value
	end
end

# A basic linked list
#
class LinkedList
	# Add the value to the front of the linked list
	def add(value)
		if @head == nil
			@head = Node.new(value)
		else
			temp = @head
			@head = Node.new(value)
			@head.next = temp
		end
	end
	
	# Remove the value at the front of the linked list
	# and return it
	def remove
		if @head != nil
			temp = @head
			@head = @head.next
			return temp.value
		end
	end
	
	# Apply the "block" to each element in the
	# linked list.  The block should expect a single
	# value.
	def each
		temp = @head
		
		while temp != nil
			yield temp.value
			temp = temp.next
		end
	end
	
	# Apply the "block" to each element in the linked list
	# with the index.  The block should expect a value and
	# and index.
	def each_with_index
		temp = @head
		index = 0
		
		while not temp.nil?
			yield(temp.value, index)
			temp = temp.next
			index = index + 1
		end
	end
	
	# return the first value in the linked list
	# where the block passed in returns true.  return
	# nil if no value returns true.
	def find
		# this call to "each" is the method defined above
		each do |v|
			return v if yield(v)  # could also be written: if yield(v) return v end (spaced appropriately)
		end
		return nil
	end
	
	# write out the contents of the linked list
	def to_s
		s = ""

		# reuse the each method defined above
		each{|v| s += v.to_s + " "}		
		return s
	end
end
